@@ -113,7 +113,7 @@ The array is used because the family has two boys.
 
 If a element has both of a text node and attributes
 or both of a text node and other child nodes,
-value of a text node is moved to '#text' like child nodes.
+value of a text node is moved to C<'#text'> like child nodes.
 
     use XML::TreePP;
     use Data::Dumper;
@@ -132,7 +132,7 @@ The result dumped is following:
         }
     };
 
-The special node name of '#text' is used because this elements
+The special node name of C<'#text'> is used because this elements
 has attribute(s) in addition to the text node.
 
 =head1 CONSTRUCTOR AND OPTIONS
@@ -188,15 +188,21 @@ default, cdata section is converted into a scalar.
 
 This option allows you to specify a HTTP_USER_AGENT string which
 is used by parsehttp() method.
-The default string is "XML-TreePP/#.##", where "#.##" is
+The default string is C<"XML-TreePP/#.##">, where C<"#.##"> is
 substituted with the version number of this library.
 
 =head2 $tpp->set( attr_prefix => '@' );
 
 This option allows you to specify a prefix character(s) which
 is inserted before each attribute names.
-The default character is '-'.
-Or set '@' to access attribute values like E4X, ECMAScript for XML.
+The default character is C<'-'>.
+Or set C<'@'> to access attribute values like E4X, ECMAScript for XML.
+Zero-length prefix C<''> is also available now.
+
+=head2 $tpp->set( text_node_key => '#text' );
+
+This option allows you to specify a hash key for text nodes.
+The default key is C<'#text'>.
 
 =head2 $tpp->set( ignore_error => 1 );
 
@@ -208,6 +214,21 @@ This option makes all errors ignored and just return.
 This module generates an XML declaration on writing an XML code per default.
 This option forces to change or leave it.
 
+=head2 $tpp->set( http_lite => $http );
+
+This option forces pasrsehttp() method to use L<HTTP::Lite> module 
+with its instance created like: C<$http = HTTP::Lite-E<gt>new();>
+
+=head2 $tpp->set( lwp_useragent => $ua );
+
+This option forces pasrsehttp() method to use L<LWP::UserAgent> module 
+with its instance created like: C<$ua = LWP::UserAgent-E<gt>new();>
+
+=head2 $tpp->set( use_ixhash => 1 );
+
+This option keeps the order for every elements appeared in XML.
+L<Tie::IxHash> module is required.
+
 =head2 $tpp->get( "option_name" );
 
 This method returns a current option value for "option_name".
@@ -232,7 +253,7 @@ $method is a method of HTTP connection: GET/POST/PUT/DELETE
 $url is an URI of a XML file.
 $body is a request body when you use POST method.
 $head is a request headers as a hash ref.
-LWP::UserAgent module or HTTP::Lite module is required to fetch a file.
+L<LWP::UserAgent> module or L<HTTP::Lite> module is required to fetch a file.
 
 =head2 $source = $tpp->write( $tree, $encode );
 
@@ -251,9 +272,9 @@ Yusuke Kawasaki, http://www.kawa.net/
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2006 Yusuke Kawasaki.  All rights reserved.  This program
-is free software; you can redistribute it and/or modify it under the same
-terms as Perl itself.
+Copyright (c) 2006-2007 Yusuke Kawasaki. All rights reserved.
+This program is free software; you can redistribute it and/or 
+modify it under the same terms as Perl itself.
 
 =cut
 
@@ -263,12 +284,13 @@ use Carp;
 use Symbol;
 
 use vars qw( $VERSION );
-$VERSION = '0.18';
+$VERSION = '0.20';
 
 my $XML_ENCODING      = 'UTF-8';
 my $INTERNAL_ENCODING = 'UTF-8';
 my $USER_AGENT        = 'XML-TreePP/'.$VERSION.' ';
 my $ATTR_PREFIX       = '-';
+my $TEXT_NODE_KEY     = '#text';
 
 sub new {
     my $package = shift;
@@ -326,16 +348,30 @@ sub write {
     my $to   = shift || $self->{output_encoding} || $XML_ENCODING;
     my $decl = $self->{xml_decl};
     $decl = '<?xml version="1.0" encoding="' . $to . '" ?>' unless defined $decl;
+
+    local $self->{__first_out};
     if ( exists $self->{first_out} ) {
         my $keys = $self->{first_out};
         $keys = [$keys] unless ref $keys;
-        $self->{__first_out} = { map { $_ => 1 } @$keys };
+        $self->{__first_out} = { map { $keys->[$_] => $_ } 0 .. $#$keys };
     }
+
+    local $self->{__last_out};
     if ( exists $self->{last_out} ) {
         my $keys = $self->{last_out};
         $keys = [$keys] unless ref $keys;
-        $self->{__last_out} = { map { $_ => 1 } @$keys };
+        $self->{__last_out} = { map { $keys->[$_] => $_ } 0 .. $#$keys };
     }
+
+    my $tnk = $self->{text_node_key};
+    local $self->{text_node_key} = $TEXT_NODE_KEY;
+    $self->{text_node_key} = $tnk if defined  $tnk;
+
+    my $apre = $self->{attr_prefix};
+    $apre = $ATTR_PREFIX unless defined  $apre;
+    local $self->{__attr_prefix_len} = length($apre);
+    local $self->{__attr_prefix_rex} = defined $apre ? qr/^\Q$apre\E/s : undef;
+
     my $text = $self->hash_to_xml( undef, $tree );
     if ( $from && $to ) {
         my $stat = $self->encode_from_to( \$text, $from, $to );
@@ -348,6 +384,7 @@ sub write {
 sub parsehttp {
     my $self = shift;
 
+    local $self->{__user_agent};
     if ( exists $self->{user_agent} ) {
         my $agent = $self->{user_agent};
         $agent .= $USER_AGENT if ( $agent =~ /\s$/s );
@@ -358,10 +395,9 @@ sub parsehttp {
 
     my $http = $self->{__http_module};
     unless ( $http ) {
-        $http = &find_http_module();
+        $http = $self->find_http_module(@_);
         $self->{__http_module} = $http;
     }
-
     if ( $http eq 'LWP::UserAgent' ) {
         return $self->parsehttp_lwp(@_);
     }
@@ -374,16 +410,46 @@ sub parsehttp {
 }
 
 sub find_http_module {
+    my $self = shift || {};
+
+    if ( exists $self->{lwp_useragent} && ref $self->{lwp_useragent} ) {
+        return 'LWP::UserAgent' if defined $LWP::UserAgent::VERSION;
+        return 'LWP::UserAgent' if &load_lwp_useragent();
+        return $self->die( "LWP::UserAgent is required: $_[1]" );
+    }
+
+    if ( exists $self->{http_lite} && ref $self->{http_lite} ) {
+        return 'HTTP::Lite' if defined $HTTP::Lite::VERSION;
+        return 'HTTP::Lite' if &load_http_lite();
+        return $self->die( "HTTP::Lite is required: $_[1]" );
+    }
+
     return 'LWP::UserAgent' if defined $LWP::UserAgent::VERSION;
     return 'HTTP::Lite'     if defined $HTTP::Lite::VERSION;
+    return 'LWP::UserAgent' if &load_lwp_useragent();
+    return 'HTTP::Lite'     if &load_http_lite();
+    return $self->die( "LWP::UserAgent or HTTP::Lite is required: $_[1]" );
+}
 
+sub load_lwp_useragent {
+    return $LWP::UserAgent::VERSION if defined $LWP::UserAgent::VERSION;
     local $@;
     eval { require LWP::UserAgent; };
-    return 'LWP::UserAgent' if defined $LWP::UserAgent::VERSION;
+    $LWP::UserAgent::VERSION;
+}
 
+sub load_http_lite {
+    return $HTTP::Lite::VERSION if defined $HTTP::Lite::VERSION;
+    local $@;
     eval { require HTTP::Lite; };
-    return 'HTTP::Lite'     if defined $HTTP::Lite::VERSION;
-    return;
+    $HTTP::Lite::VERSION;
+}
+
+sub load_tie_ixhash {
+    return $Tie::IxHash::VERSION if defined $Tie::IxHash::VERSION;
+    local $@;
+    eval { require Tie::IxHash; };
+    $Tie::IxHash::VERSION;
 }
 
 sub parsehttp_lwp {
@@ -393,11 +459,16 @@ sub parsehttp_lwp {
     my $body   = shift;
     my $header = shift;
 
-    my $ua = LWP::UserAgent->new();
-    $ua->timeout(10);
-    $ua->env_proxy();
+    my $ua = $self->{lwp_useragent} if exists $self->{lwp_useragent};
+    if ( ! ref $ua ) {
+        $ua = LWP::UserAgent->new();
+        $ua->timeout(10);
+        $ua->env_proxy();
+        $ua->agent( $self->{__user_agent} ) if defined $self->{__user_agent};
+    } else {
+        $ua->agent( $self->{__user_agent} ) if exists $self->{user_agent};
+    }
 
-    $ua->agent( $self->{__user_agent} ) if defined $self->{__user_agent};
     my $req = HTTP::Request->new( $method, $url );
     my $ct = 0;
     if ( ref $header ) {
@@ -465,11 +536,26 @@ sub parse {
             return $self->die( "Unsupported encoding: $from" ) unless $stat;
         }
     }
+
+    local $self->{__force_array};
     if ( exists $self->{force_array} ) {
         my $force = $self->{force_array};
         $force = [$force] unless ref $force;
         $self->{__force_array} = { map { $_ => 1 } @$force };
     }
+
+    my $tnk = $self->{text_node_key};
+    local $self->{text_node_key} = $TEXT_NODE_KEY;
+    $self->{text_node_key} = $tnk if defined  $tnk;
+
+    my $apre = $self->{attr_prefix};
+    local $self->{attr_prefix} = $ATTR_PREFIX;
+    $self->{attr_prefix} = $apre if defined  $apre;
+
+    if ( exists $self->{use_ixhash} && $self->{use_ixhash} ) {
+        return $self->die( "Tie::IxHash is required." ) unless &load_tie_ixhash();
+    }
+
     my $flat = $self->xml_to_flat($textref);
     my $tree = $self->flat_to_tree( $flat, '' );
     wantarray ? ( $tree, $$textref ) : $tree;
@@ -479,7 +565,7 @@ sub xml_to_flat {
     my $self    = shift;
     my $textref = shift;    # reference
     my $flat    = [];
-    my $prefix = $self->{attr_prefix} || $ATTR_PREFIX;
+    my $prefix = $self->{attr_prefix};
     while ( $$textref =~ m{
         ([^<]*) <
         ((
@@ -562,6 +648,10 @@ sub flat_to_tree {
     my $tree   = {};
     my $text   = [];
 
+    if ( exists $self->{use_ixhash} && $self->{use_ixhash} ) {
+        tie( %$tree, 'Tie::IxHash' );
+    }
+
     while ( scalar @$source ) {
         my $node = shift @$source;
         if ( !ref $node || UNIVERSAL::isa( $node, "SCALAR" ) ) {
@@ -585,7 +675,7 @@ sub flat_to_tree {
                 }
                 elsif ( defined $child ) {
                     # some attributes and text node
-                    $elem->{'#text'} = $child;
+                    $elem->{$self->{text_node_key}} = $child;
                 }
             }
             else {
@@ -615,7 +705,7 @@ sub flat_to_tree {
         }
         if ( scalar keys %$tree ) {
             # some child nodes and also text node
-            $tree->{'#text'} = $text;
+            $tree->{$self->{text_node_key}} = $text;
         }
         else {
             # only text node without child nodes
@@ -631,17 +721,20 @@ sub hash_to_xml {
     my $hash      = shift;
     my $out       = [];
     my $attr      = [];
-    my $allkeys   = [ sort keys %$hash ];
-    my $firstkeys = [ grep { $self->{__first_out}->{$_} } @$allkeys ] if ref $self->{__first_out};
-    my $lastkeys = [ grep { $self->{__last_out}->{$_} } @$allkeys ] if ref $self->{__last_out};
-    $allkeys = [ grep { !$self->{__first_out}->{$_} } @$allkeys ] if ref $self->{__first_out};
-    $allkeys = [ grep { !$self->{__last_out}->{$_} } @$allkeys ] if ref $self->{__last_out};
-    my $prefix = $self->{attr_prefix} || $ATTR_PREFIX;
-    $prefix = "\Q$prefix\E";
+    my $allkeys   = [ keys %$hash ];
+    my $fo = $self->{__first_out} if ref $self->{__first_out};
+    my $lo = $self->{__last_out}  if ref $self->{__last_out};
+    my $firstkeys = [ sort { $fo->{$a} <=> $fo->{$b} } grep { exists $fo->{$_} } @$allkeys ] if ref $fo;
+    my $lastkeys  = [ sort { $lo->{$a} <=> $lo->{$b} } grep { exists $lo->{$_} } @$allkeys ] if ref $lo;
+    $allkeys = [ grep { ! exists $fo->{$_} } @$allkeys ] if ref $fo;
+    $allkeys = [ grep { ! exists $lo->{$_} } @$allkeys ] if ref $lo;
+    $allkeys = [ sort @$allkeys ] unless $self->{use_ixhash};
+    my $prelen = $self->{__attr_prefix_len};
+    my $pregex = $self->{__attr_prefix_rex};
 
     foreach my $loopkey ( $firstkeys, $allkeys, $lastkeys ) {
         next unless ref $loopkey;
-        foreach my $key ( grep { !/^$prefix/ } @$loopkey ) {
+        foreach my $key ( grep { ! $prelen || $_ !~ $pregex } @$loopkey ) {
             my $val = $hash->{$key};
             if ( !defined $val ) {
                 push( @$out, "<$key />" );
@@ -663,8 +756,9 @@ sub hash_to_xml {
                 push( @$out, $child );
             }
         }
-        foreach my $key ( grep { /^$prefix/ } @$loopkey ) {
-            my $name = ( $key =~ /^$prefix(.*)$/s )[0];
+    
+        foreach my $key ( grep { $prelen && $_ =~ $pregex } @$loopkey ) {
+            my $name = substr( $key, $prelen );
             my $val = &xml_escape( $hash->{$key} );
             push( @$attr, ' ' . $name . '="' . $val . '"' );
         }
@@ -721,7 +815,7 @@ sub scalaref_to_cdata {
     my $name = shift;
     my $ref  = shift;
     my $text = '<![CDATA[' . $$ref . ']]>';
-    $text = "<$name>$text</$name>\n" if ( $name ne '#text' );
+    $text = "<$name>$text</$name>\n" if ( $name ne $self->{text_node_key} );
     $text;
 }
 
@@ -731,7 +825,7 @@ sub scalar_to_xml {
     my $scalar = shift;
     my $copy   = $scalar;
     my $text   = &xml_escape($copy);
-    $text = "<$name>$text</$name>\n" if ( $name ne '#text' );
+    $text = "<$name>$text</$name>\n" if ( $name ne $self->{text_node_key} );
     $text;
 }
 
